#include <avr/io.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
//#include <string.h>

#include "delay.h"
#include "adc.h"
#include "data.h"
#include "timer.h"

// Fuse byte settings:
// Fuse extended byte: 0xff (default)
// Fuse high byte: 0xdd (BOD enabled, 2,5-2,9V)
// Fuse low byte: 0xd2 (RC-On-Chip 8MHz, long startup, no clock division)

int main(void)
{
   // Setup the analog to digital converter
   setupADC();

   // Setup timers
   setupTimers();
   
   // Setup Watchdog for 2 seconds timeout
   wdt_enable(WDTO_2S);

   // Setup ports for output
   DDRB |= (1<<PB0) | (1<<PB1) | (1<<PB2);
   
   sei();
   
   while(1)
     {
	static uint8_t lambda_high_values,
	  lambda_low_values,
	  lambda_error_values;
	uint8_t i;

	// calculate lambda value - when lambda table has been filled
	if(flags & 1<<LAMBDA_TABLE_FILLED)
	  {	     
	     lambda_high_values = 0;
	     lambda_low_values = 0;
	     lambda_error_values = 0;
	     for(i=0; i<LAMBDA_TABLE_SIZE; i++)
	       {
		  if(lambda[i] < 100) // lambda voltage < 0.5V
		    {		       
		       lambda_low_values++;
		    }		  
		  else 
		    {
		       if((lambda[i] > 100) && (lambda[i] < 200)) // 0.5V < lambda_voltage < 1V
			 {
			    lambda_high_values++;
			 }
		       else
			 {			    
			    lambda_error_values++;
			 }
		    }		  
	       }
	     flags &= ~(1<<LAMBDA_TABLE_FILLED);
	  }	
	// show current lambda value
	if(lambda_error_values > (LAMBDA_TABLE_SIZE / 4))
	  // too many values out of range
	  {		  
	     // turn off green LED
	     PORTB &= !(1<<PB1);
	     // turn on red LED and yellow LED
	     PORTB |= (1<<PB0) | (1<<PB2);
	  }
	else
	  {
	     if(lambda_high_values < (lambda_low_values / 4))
	       // more than four times as many low values than high values
	       // -> Engine running lean
	       {
		  // turn off green LED and yellow LED
		  PORTB &= !( (1<<PB1) | (1<<PB2) );
		  // turn on red LED
		  PORTB |= (1<<PB0);
	       }	     
	     else 
	       {
		  if(lambda_low_values < (lambda_high_values / 4))
		    // more than four times as many high values than low values
		    // -> Engine running rich
		    {			    
		       // turn off green LED and red LED
		       PORTB &= !( (1<<PB0) | (1<<PB1) );
		       // turn on yellow LED
		       PORTB |= (1<<PB2);
		    }		       
		  else
		    {			    
		       // turn off red LED and yellow LED
		       PORTB &= !( (1<<PB0) | (1<<PB2) );
		       // turn on green LED
		       PORTB |= (1<<PB1);
		    }
	       }
	  }

	// reset the watchdog timer
	wdt_reset();
     }
   
}
